/*
 * main.cpp
 *
 * Created by Slamtec at 2021-07-06
 * Copyright 2021 (c) Shanghai Slamtec Co., Ltd.
 */
#include <regex>
#include <rpos/robot_platforms/slamware_core_platform.h>
#include <rpos/core/metadata.h>
#include <boost/thread.hpp>

using namespace rpos::robot_platforms;
using namespace rpos::features::motion_planner;
using namespace rpos::features::artifact_provider;


std::string ip_address = "";
const char *ip_regex = "\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}";
const double pi = 3.1415927;

void ShowHelp(std::string app_name) {
	std::cout << "SLAMWARE console demo." << std::endl << 
        "Usage: " << app_name << " <slamware_address>" << std::endl;
}

bool ParseCommandLine(int argc, const char *argv[]) {
	bool opt_show_help = false;
	for (int pos = 1; pos < argc; ++pos) {
		const char *current = argv[pos];
		if (strcmp(current, "-h") == 0)
			opt_show_help = true;
	    else 
			ip_address = current;		
	}
	std::regex reg(ip_regex);
	if (!opt_show_help && !std::regex_match(ip_address, reg)) 
		opt_show_help = true;
	if (opt_show_help) {
		ShowHelp("forbidden_area");
		return false;
	}
	return true;
}

int main(int argc, const char *argv[]) {
	if (!ParseCommandLine(argc, argv)) return 1;
	std::cout << "Connecting SDP @ " << ip_address << "..."<< std::endl;
	try {
		SlamwareCorePlatform sdp = SlamwareCorePlatform::connect(ip_address, 1445);
		std::cout <<"SDK Version: " << sdp.getSDKVersion() << std::endl;
		std::cout <<"SDP Version: " << sdp.getSDPVersion() << std::endl;
		
		/*add a forbidden area.
		start point(0,1),
		end point(3,1),
		width=2*2,
		escape region size=1 */
		rpos::core::ORectangleF area(rpos::core::Vector2f(0,1),rpos::core::Vector2f(3,1),2);  
		
		rpos::core::Metadata metadata;
		std::string escapedistance="escape_distance";
		std::string valueofescapedistance="1";
		metadata.set(escapedistance,valueofescapedistance);

		rpos::features::artifact_provider::RectangleArea rectanglearea;
		rectanglearea.id=1;
		rectanglearea.metadata=metadata;
		rectanglearea.area=area;
		rectanglearea.usage=ArtifactUsageForbiddenArea;
		
		
		std::cout <<"Adding a forbidden area..."<< std::endl;
		sdp.addRectangleArea(ArtifactUsageForbiddenArea,rectanglearea);

		std::cout <<"Clearing all the forbidden area after 10s ..."<< std::endl;
		boost::this_thread::sleep_for(boost::chrono::milliseconds(10000)); //sleep 10000ms
		sdp.clearRectangleAreas(ArtifactUsageForbiddenArea);

	}
	catch (const rpos::system::detail::ExceptionBase& e) {		
		std::cout << e.what() << std::endl;
		return 1;	
	}
	return 0;
} 