/*
 * main.cpp
 *
 * Created by Slamtec at 2021-07-06
 * Copyright 2021 (c) Shanghai Slamtec Co., Ltd.
 */
#include <iostream>
#include <boost/thread.hpp>
#include <regex>

#include <rpos/robot_platforms/slamware_core_platform.h>

std::string ip_address = "";
const char *ip_regex = "\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}";
rpos::robot_platforms::SlamwareCorePlatform sdp;

void ShowHelp(std::string app_name) {
	std::cout << "SLAMWARE console demo." << std::endl << 
        "Usage: " << app_name << " <slamware_address>" << std::endl;
}

bool ParseCommandLine(int argc, const char *argv[]) {
    bool opt_show_help = false;
    for (int pos = 1; pos < argc; ++pos) {
        const char *current = argv[pos];
        if (strcmp(current, "-h") == 0)
            opt_show_help = true;
		else 
            ip_address = current;
    }
    std::regex reg(ip_regex);
    if (!opt_show_help && !std::regex_match(ip_address, reg)) 
        opt_show_help = true;
    if (opt_show_help) {
        ShowHelp("get_system_event");
        return false;
    }
    return true;
}

int main(int argc,const char* argv[])
{
	if (!ParseCommandLine(argc , argv)) return 1;
	std::cout << "Connecting SDP @ " << ip_address << "..."<< std::endl;

	try {
		sdp = rpos::robot_platforms::SlamwareCorePlatform::connect(ip_address, 1445);
        std::cout <<"SDK Version: " << sdp.getSDKVersion() << std::endl;
        std::cout <<"SDP Version: " << sdp.getSDPVersion() << std::endl;
		//create ptr of SystemEventProvider
		boost::shared_ptr<rpos::robot_platforms::objects::SystemEventProvider> p_SystemEventProvider = sdp.createSystemEventProvider();

        //read event
		while(true){
			//create a temp vecter to save events
			std::vector<rpos::core::DiagnosisInfoInternalSystemEvent> v_event;
			
			if(!p_SystemEventProvider->readEvents(v_event))
			{
				p_SystemEventProvider = sdp.createSystemEventProvider();
				continue;
			}

			if(!v_event.empty())
			{
				std::cout<< "Current event:" << std::endl;
				for (auto iter = v_event.cbegin(); iter != v_event.cend(); iter++)
				{
					std::cout<< iter->internalSystemEvent << std::endl; 
				}
			}	
			boost::this_thread::sleep_for(boost::chrono::milliseconds(100)); //sleep 100ms
        }
		return 0;
	}

    catch(const rpos::system::detail::ExceptionBase& e){
        std::cout << e.what() << std::endl; 
        return 1;
    }
}